package chat.client;

import chat.common.Helper;
import java.util.HashMap;
import java.rmi.*;
import java.rmi.server.*;

/**
 * Gerencia a conexo de um usurio com uma sala de chat, desde o lado do cliente.
 * Funciona como um Bridge.
 *
 * @author Jorge F. Del Teglia
 */
class ChatClientImpl extends UnicastRemoteObject
  implements ChatClient
////////////////////////////////////////////////
{
  private ChatClient clientListener;


  /**
   * Inicializa o objeto especificando quem processar as requicises.
   *
   * @param clientListener
   */
  public ChatClientImpl( ChatClient clientListener )
    throws RemoteException
  //////////////////////////////////////////////////
  {
    this.clientListener = clientListener;
  }


  /**
   * Recebe uma operacao do usurio especificado.
   */
  public void receiveOperation( GenericOperation gop )
    throws RemoteException
  {
    // Delegation.
    this.clientListener.receiveOperation( gop );
  }


  public void userEnteredRoom( String userName )
    throws RemoteException
  //////////////////////////////////////////////
  {
    // Delegation.
    this.clientListener.userEnteredRoom( userName );
  }


  public void userLeavedRoom( String userName )
    throws RemoteException
  /////////////////////////////////////////////
  {
    // Delegation.
    this.clientListener.userLeavedRoom( userName );
  }


  /**
   * S para checar estado.
   */
  public void checkAlive()
    throws RemoteException
  ////////////////////////
  {
    // Nada para ser feito.
  }


  public String receiveInviteToHCSession( String sessionName )
    throws RemoteException
  {
    // Delegation.
    return this.clientListener.receiveInviteToHCSession( sessionName );
  }


} // class ChatClientImpl


