package chat.client;

import javax.swing.*;
import javax.swing.border.*;
import java.awt.*;
import java.awt.event.*;
import chat.common.Helper;
import chat.server.*;
import java.rmi.RemoteException;

/**
 * Um cliente do servidor de chat.
 */
public class ClientFrame extends JFrame
  implements ActionListener, ChatClient
///////////////////////////////////////
{
  private JTextArea msgHistoryTextArea;
  private JTextField newMsgTextField;
  private JButton sendButton;
  private JComboBox destNameComboBox;
  private JComboBox roomNameComboBox;
  private JTextField userNameTextField;
  private JButton connectButton;
  private JButton disconnectButton;
  private JButton hyperChatButton;
  private JCheckBox broadcastCheckBox;
  private Client client;
  
  // @todo: change architecture.
  private HCFrame hcFrame;


  public ClientFrame( String args[] )
    throws Exception
  ///////////////////////////////////
  {
    super( "HyperChat Client - [Jorge F. Del Teglia - July 2002]" );

    // Cria o model.
    this.client = new Client( this, args );
    this.hcFrame = null;

    getContentPane().add( createMainContainer(), BorderLayout.CENTER );
    setEnablings();

    pack();
    setDefaultCloseOperation( JFrame.EXIT_ON_CLOSE );

    // Asegur que ao fechar a janela, desconete do servidor.
    addWindowListener( new WindowAdapter()
    {
      public void windowClosing( WindowEvent e )
      {
        try
        {
          doDisconnect();
        }
        catch ( Exception ex )
        {
        }
      }
    } );
  }


  public void actionPerformed( ActionEvent ae )
  ////////////////////////////////////////////
  {
    Object src = ae.getSource();

    try
    {
      if ( src == this.connectButton )
      {
        doConnect();
      }
      else if ( src == this.disconnectButton )
      {
        doDisconnect();
      }
      else if ( src == this.sendButton )
      {
        doSendMessage();
      }
      else if ( src == this.broadcastCheckBox )
      {
        ;
      }
      else if ( src == this.hyperChatButton )
      {
        doStartHyperChatSession();
      }

      setEnablings();
    }
    catch ( ChatRoomDoesntExistException e )
    {
      JOptionPane.showMessageDialog( this, e.getMessage(), "Erro", JOptionPane.ERROR_MESSAGE );
    }
    catch ( UserAlreadyInRoomException e )
    {
      JOptionPane.showMessageDialog( this, e.getMessage(), "Erro", JOptionPane.ERROR_MESSAGE );
    }
    catch ( UserNotInRoomException e )
    {
      JOptionPane.showMessageDialog( this, e.getMessage(), "Erro", JOptionPane.ERROR_MESSAGE );
    }
    catch ( RemoteException e )
    {
      JOptionPane.showMessageDialog( this, e.getMessage(), "Erro", JOptionPane.ERROR_MESSAGE );
    }
    catch ( Exception e )
    {
      JOptionPane.showMessageDialog( this, e.getMessage(), "Erro", JOptionPane.ERROR_MESSAGE );
    }
  }


  private void doStartHyperChatSession()
    throws Exception
  {
    if ( this.hcFrame != null )
    {
      throw new IllegalStateException( "Verso no suporta mais de uma sesso simultnea de hyperchat" );
    }

    // Abre dialogo para escolha de participantes.
    // @todo hardcoded; use results from dialog.
    String[] participants = this.client.getUserNamesInRoom();

    // Inicia sesso sem ningueim. Os outros vo se sumar logo.
    // @todo: estender para suportar multiplas sesses de HC simultneas.
    openHCFrame();
    
    // Notificar todos (menos eu) e convidar a participar.
    for ( int i = 0; i < participants.length; i++  )
    {
      if ( !participants[ i ].equals( this.client.getUserName() ) )
      {
        //this.client.sendOperation( invitation, participants[ i ] );
        String answer = this.client.inviteToHCSession( "HC sala 1", participants[ i ] );

        if ( answer != null )
        {
          JOptionPane.showMessageDialog( this.hcFrame, 
            "O usurio " + participants[ i ] + " no aceitou o convite.\n" + answer, 
            "Convite no aceito", JOptionPane.INFORMATION_MESSAGE );
        }
        else
        {
          JOptionPane.showMessageDialog( this.hcFrame, 
            "O usurio " + participants[ i ] + " aceitou o convite.\n", 
            "Convite aceito!", JOptionPane.INFORMATION_MESSAGE );

          this.hcFrame.registerParticipant( participants[ i ] );
        }
      } // if
    } // for i
  }


  private void openHCFrame()
    throws Exception
  {
    this.hcFrame = new HCFrame( this.client, this );
    this.hcFrame.setLocation( this.getLocation() );
    this.hcFrame.setVisible( true );
  }

/*
  void processInvitationAnswer( InvitationAnswerOp iaop )
  {
    this.hcFrame.processInvitationAnswer( iaop );
  }
*/

/*
  void processInvitation( InvitationOp iop )
    throws Exception
  {
    if ( this.hcFrame == null )
    {
      // Aceita convite? @todo
      boolean accepted = true;
      if ( accepted )
      {
        openHCFrame();
        
        // Aceita convite.
        this.client.sendOperation( new InvitationAnswerOp( this.client.getUserName(), true, null  ), iop.getOperationCreatorName() );
      }
      else
      {
        // Rechaza convite.
        this.client.sendOperation( new InvitationAnswerOp( this.client.getUserName(), false, "No desejo participar"  ), iop.getOperationCreatorName() );
      }
    }
    else
    {
      // Rechaza convite por j estar numa outra sesso HC.
      this.client.sendOperation( new InvitationAnswerOp( this.client.getUserName(), false, "J estou numa sesso Hyperchat"  ), iop.getOperationCreatorName() );
    }
  }
*/

  public String receiveInviteToHCSession( String sessionName )
    throws RemoteException
  {
    try
    {
      if ( this.hcFrame == null )
      {
        // Aceita convite? @todo
        boolean accepted = true;
        if ( accepted )
        {
          openHCFrame();

          // Aceita convite.
          return null;
        }
        else
          return "No desejo participar";  // Rechaza convite.
      }
      else
        return "J estou numa sesso Hyperchat"; // Rechaza convite por j estar numa outra sesso HC.
    }
    catch ( Exception e )
    {
      e.printStackTrace();
      return "Error abrindo o HCFrame: " + e.getMessage();
    }
  }


  void terminateHyperChatSession()
  {
    this.hcFrame = null;
    setEnablings();
  }


  private void doSendMessage()
    throws UserNotInRoomException, RemoteException
  ///////////////////////////////
  {
    String message = newMsgTextField.getText();
    GenericOperation gop = new WriteMsgOp( this.client.getUserName(), message );

    if ( this.broadcastCheckBox.isSelected() )
    {
      this.client.broadcastOperation( gop );
    }
    else
    {
      this.client.sendOperation( gop, this.destNameComboBox.getSelectedItem().toString() );
    }

    // Inform here.
    displayInHistory( this.client.getUserName(), message );

    // Limpa o mensagem enviado.
    this.newMsgTextField.setText( "" );

    this.newMsgTextField.requestFocus();
  }


  private void doDisconnect()
  throws ChatRoomDoesntExistException, RemoteException
  /////////////////////////////////////
  {
    this.client.disconnect();
    this.msgHistoryTextArea.setText( "" );
  }


  private void doConnect()
  throws ChatRoomDoesntExistException, UserAlreadyInRoomException, UserNotInRoomException, RemoteException
  /////////////////////////////////////////////////////////////////////////////////////////
  {
    String userName = this.userNameTextField.getText().trim();
    if ( userName.length() > 0 )
    {
      this.client.connect( this.roomNameComboBox.getModel().getSelectedItem().toString(),
                           this.userNameTextField.getText() );

      // Carrega os nomes de usurio na sala.
      populateUsers();
    } // if
    else
    {
      JOptionPane.showMessageDialog( this, "Nome invlido", "Erro", JOptionPane.ERROR_MESSAGE );
    }
  }



  //
  // -- ChatClientOperations --------------------------------------------------
  //


  /**
   * Recebe uma operacao do usurio especificado.
   */
  public void receiveOperation( GenericOperation gop )
    throws RemoteException
  {
    System.out.println( "Executing op in dest client sent from " + gop.getOperationCreatorName() );

    if ( gop.isGraphicOperation() )
    {
      // Forward to HyperchatFrame
      if ( this.hcFrame != null )
      {
        this.hcFrame.receiveOperation( gop );
      }
      else
        System.out.println( "Losed operation - no HyperChat session active. Source: " + gop.getOperationCreatorName() );
    }
    else
    {
      // Process here.
      gop.execute( this );
    }
  }


  public void userEnteredRoom( String userName )
  //////////////////////////////////////////////
  {
    displayInHistory( "System", "[" + userName + "] entrou na sala." );
    populateUsers();
    setEnablings();
  }


  public void userLeavedRoom( String userName )
  /////////////////////////////////////////////
  {
    displayInHistory( "System", "[" + userName + "] saiu da sala." );
    populateUsers();
    setEnablings();
  }


  public void checkAlive()
  ///////////////////////
  {
    // Nada para fazer.
  }


  //
  // -- Internal methods ------------------------------------------------------
  //

  private void populateUsers()
  ////////////////////////////
  {
    java.util.Vector userNames = new java.util.Vector( java.util.Arrays.asList( this.client.getUserNamesInRoom() ) );
    userNames.remove( this.client.getUserName() );
    this.destNameComboBox.setModel( new DefaultComboBoxModel( userNames ) );
  }


  void displayInHistory( String src, String msg )
  ///////////////////////////////////////////////////////
  {
    this.msgHistoryTextArea.append( "[" + src + "] >> " + msg );
    this.msgHistoryTextArea.append( "\n" );
    this.msgHistoryTextArea.setCaretPosition( this.msgHistoryTextArea.getDocument().getLength() );
  }


  /**
   * Enable/disable controls depending on the state.
   */
  private void setEnablings()
  ///////////////////////////
  {
    boolean isConnected = this.client.isConnected();

    this.userNameTextField.setEnabled( !isConnected );
    this.roomNameComboBox.setEnabled( !isConnected );
    this.connectButton.setEnabled( !isConnected );
    this.disconnectButton.setEnabled( isConnected );
    this.msgHistoryTextArea.setEnabled( isConnected );
    this.newMsgTextField.setEnabled( isConnected );
    this.sendButton.setEnabled( isConnected && this.destNameComboBox.getModel().getSize() > 0 );
    this.broadcastCheckBox.setEnabled( isConnected );
    this.destNameComboBox.setEnabled( isConnected && !this.broadcastCheckBox.isSelected() );
    this.hyperChatButton.setEnabled( isConnected && (this.hcFrame == null) );
  }


  private Container createMainContainer()
  ///////////////////////////////////////
  {
    JPanel mainPanel = new JPanel( new BorderLayout() );
    mainPanel.add( createConnectionControls(), BorderLayout.NORTH );
    mainPanel.add( createChatArea(), BorderLayout.CENTER );
    mainPanel.add( createChatControls(), BorderLayout.SOUTH );

    return mainPanel;
  }


  private Component createChatArea()
  //////////////////////////////////
  {
    // --
    this.msgHistoryTextArea = new JTextArea( 20, 40 );
    this.msgHistoryTextArea.setBackground( new Color( 228, 185, 126 ) );
    this.msgHistoryTextArea.setEditable( false );
    JScrollPane areaScrollPane = new JScrollPane( msgHistoryTextArea,
    JScrollPane.VERTICAL_SCROLLBAR_ALWAYS,
    JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED );
    areaScrollPane.setBorder( BorderFactory.createBevelBorder( BevelBorder.RAISED ) );

    return areaScrollPane;
  }


  private Container createChatControls()
  //////////////////////////////////////
  {
    this.newMsgTextField = new JTextField( "Digite aqui sua mensagem" );

    // --
    this.sendButton = new JButton( "Enviar" );
    this.sendButton.addActionListener( this );
    getRootPane().setDefaultButton( this.sendButton );

    // --
    this.destNameComboBox = new JComboBox( new DefaultComboBoxModel() );

    // --
    this.broadcastCheckBox = new JCheckBox( "Broadcast", false );
    this.broadcastCheckBox.addActionListener( this );

    // -- Cria o panel de cpontroles de chat.
    Box chatControls = new Box( BoxLayout.X_AXIS );
    chatControls.add( Box.createRigidArea( new Dimension( 3, 0 ) ) );
    chatControls.add( this.newMsgTextField );
    chatControls.add( this.sendButton );
    chatControls.add( Box.createRigidArea( new Dimension( 10, 0 ) ) );
    chatControls.add( new JLabel( "To:" ) );
    chatControls.add( Box.createRigidArea( new Dimension( 3, 0 ) ) );
    chatControls.add( this.destNameComboBox );
    chatControls.add( Box.createRigidArea( new Dimension( 3, 0 ) ) );
    chatControls.add( this.broadcastCheckBox );

    return encloseComponent( chatControls );
  }


  private Container createConnectionControls()
  ////////////////////////////////////////////
  {
    // Cria o panel de controles de conexo.
    this.userNameTextField = new JTextField( 10 );
    this.connectButton = new JButton( "Conectar" );
    this.connectButton.addActionListener( this );

    // --
    this.disconnectButton = new JButton( "Desconectar" );
    this.disconnectButton.addActionListener( this );

    // --
    this.roomNameComboBox = new JComboBox( this.client.getRoomNames() );

    // --
    this.hyperChatButton  = new JButton( "HyperChat" );
    this.hyperChatButton.addActionListener( this );


    Box connectionControls = new Box( BoxLayout.X_AXIS );

    connectionControls.add( Box.createRigidArea( new Dimension( 3, 0 ) ) );
    connectionControls.add( new JLabel( "Nome de usurio:" ) );
    connectionControls.add( Box.createRigidArea( new Dimension( 3, 0 ) ) );
    connectionControls.add( this.userNameTextField );
    connectionControls.add( Box.createRigidArea( new Dimension( 20, 0 ) ) );

    connectionControls.add( new JLabel( "Sala:" ) );
    connectionControls.add( Box.createRigidArea( new Dimension( 3, 0 ) ) );
    connectionControls.add( this.roomNameComboBox );
    connectionControls.add( Box.createRigidArea( new Dimension( 20, 0 ) ) );

    connectionControls.add( Box.createHorizontalGlue() );
    connectionControls.add( this.connectButton );
    connectionControls.add( Box.createRigidArea( new Dimension( 5, 0 ) ) );
    connectionControls.add( this.disconnectButton );
    connectionControls.add( Box.createRigidArea( new Dimension( 20, 0 ) ) );
    connectionControls.add( this.hyperChatButton );

    return encloseComponent( connectionControls );
  }


  private Container encloseComponent( Component c )
  /////////////////////////////////////////////////
  {
    JPanel panel = new JPanel();
    panel.setLayout( new BoxLayout( panel, BoxLayout.Y_AXIS ) );
    panel.add( c );
    panel.setBorder( BorderFactory.createBevelBorder( BevelBorder.RAISED ) );

    return panel;
  }


  public static void main( String args[] )
  ////////////////////////////////////////
  {
    try
    {
      new ClientFrame( args ).setVisible( true );
    }
    catch ( Exception e )
    {
      e.printStackTrace();
    }
  }
} // class ClientFrame


class AutoLoggofWindowListener extends WindowAdapter
{

  /**
   * Invoked when a window is in the process of being closed.
   * The close operation can be overridden at this point.
   */
  public void windowClosing( WindowEvent e )
  {
    System.out.println( "closing..." );
  }
}

