package chat.client;

import javax.swing.*;
import javax.swing.border.*;
import java.awt.*;
import java.awt.event.*;
import chat.common.Helper;
import chat.server.*;
import java.rmi.RemoteException;

/**
 * Um cliente do servidor de chat.
 */
public class HCFrame extends JFrame
  implements ActionListener
///////////////////////////////////////
{
  private Client      client;
  private ClientFrame clientFrame;
  private JPanel      drawPanel;
  private JButton     testButton;
  private JList       participantsList;
  private DefaultListModel   model;



  public HCFrame( Client client, ClientFrame clientFrame )
    throws Exception
  ///////////////////////////////////
  {
    super( "HyperChat Session" );

    this.client = client;
    this.clientFrame = clientFrame;
    getContentPane().add( createMainContainer(), BorderLayout.CENTER );
    setEnablings();

    pack();
    setDefaultCloseOperation( JFrame.DISPOSE_ON_CLOSE );
    
    final ClientFrame finalClientFrame = clientFrame;

    // Asegur que ao fechar a janela, desconete da sessao.
    addWindowListener( new WindowAdapter()
    {
      public void windowClosing( WindowEvent e )
      {
        try
        {
          finalClientFrame.terminateHyperChatSession();
        }
        catch ( Exception ex )
        {
        }
      }
    } );


    // -- @todo
    //this.client.doStartHyperChatSession();
  }


  public Dimension getPreferredSize()
  {
    return new Dimension( 400, 300 );
  }


  void receiveOperation( GenericOperation gop )
  {
    gop.execute( this.drawPanel.getGraphics() );
  }

/*
  void processInvitationAnswer( String iaop )
  {
    if ( iaop != null )
    {
      JOptionPane.showMessageDialog( this, 
        "O usurio " + iaop.getOperationCreatorName() + " no aceitou o convite.\n" + iaop.getReason(), 
        "Convite no aceito", JOptionPane.INFORMATION_MESSAGE );
    }
    else
    {
      System.out.println( "O usurio " + iaop.getOperationCreatorName() + " aceitou o convite.\n" );
      JOptionPane.showMessageDialog( this, 
        "O usurio " + iaop.getOperationCreatorName() + " aceitou o convite.\n", 
        "Convite aceito!", JOptionPane.INFORMATION_MESSAGE );
    }
  }
*/


  public void actionPerformed( ActionEvent ae )
  ////////////////////////////////////////////
  {
    Object src = ae.getSource();

    try
    {
      if ( src == this.testButton )
      {
        this.client.testHyperChatSession();
      }
    }
    catch ( Exception e )
    {
      JOptionPane.showMessageDialog( this, e.getMessage(), "Erro", JOptionPane.ERROR_MESSAGE );
    }
  }


  //
  // -- Internal methods ------------------------------------------------------
  //

  /**
   * Enable/disable controls depending on the state.
   */
  private void setEnablings()
  ///////////////////////////
  {
  }


  private Container createMainContainer()
  ///////////////////////////////////////
  {
    JPanel mainPanel = new JPanel( new BorderLayout() );
    mainPanel.setBackground( Color.yellow.brighter() );
    mainPanel.add( createConnectionControls(), BorderLayout.NORTH );
    mainPanel.add( createDrawArea(), BorderLayout.CENTER );
    mainPanel.add( createChatControls(), BorderLayout.SOUTH );
    mainPanel.add( createParticipantsList(), BorderLayout.EAST );

    return mainPanel;
  }


  private JPanel createParticipantsList()
  {
    JPanel p = new JPanel( new BorderLayout() );
    this.model = new DefaultListModel();
    this.participantsList = new JList( this.model );
    p.add( this.participantsList, BorderLayout.CENTER );

    return p;
  }


  void registerParticipant( String participantName )
  {
    this.model.addElement( participantName );
  }


  private Component createDrawArea()
  //////////////////////////////////
  {
    // --
    this.drawPanel = new JPanel();
    JScrollPane areaScrollPane = new JScrollPane( this.drawPanel,
                                    JScrollPane.VERTICAL_SCROLLBAR_ALWAYS,
                                    JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED );
    areaScrollPane.setBorder( BorderFactory.createBevelBorder( BevelBorder.RAISED ) );

    return areaScrollPane;
  }


  private Container createChatControls()
  //////////////////////////////////////
  {
    Box chatControls = new Box( BoxLayout.X_AXIS );
    chatControls.add( Box.createRigidArea( new Dimension( 3, 0 ) ) );
    this.testButton = new JButton( "Test" );
    this.testButton.addActionListener( this );
    chatControls.add( this.testButton  );

    return encloseComponent( chatControls );
  }


  private Container createConnectionControls()
  ////////////////////////////////////////////
  {
    // Cria o panel de controles de conexo.
    Box connectionControls = new Box( BoxLayout.X_AXIS );

    connectionControls.add( Box.createRigidArea( new Dimension( 3, 0 ) ) );
    connectionControls.add( new JLabel( "Nome de usurio: " ) );
    JLabel userNameLabel = new JLabel( this.client.getUserName() );
    userNameLabel.setForeground( Color.black );
    connectionControls.add( userNameLabel );

    connectionControls.add( Box.createRigidArea( new Dimension( 8, 0 ) ) );
    connectionControls.add( new JLabel( "Sala: " ) );
    JLabel roomNameLabel = new JLabel( this.client.getCurrentRoomName() );
    connectionControls.add( roomNameLabel );
    roomNameLabel.setForeground( Color.black );
    connectionControls.add( Box.createGlue() );

    return encloseComponent( connectionControls );
  }


  private Container encloseComponent( Component c )
  /////////////////////////////////////////////////
  {
    JPanel panel = new JPanel();
    panel.setLayout( new BoxLayout( panel, BoxLayout.Y_AXIS ) );
    panel.add( c );
    panel.setBorder( BorderFactory.createBevelBorder( BevelBorder.RAISED ) );

    return panel;
  }

} // class HCFrame

