package chat.server;

import chat.common.Constants;
import chat.common.Helper;
import java.io.*;
import java.rmi.*;
import java.rmi.registry.*;
import java.rmi.server.*;

/**
 * O servidor de chat.
 * @author Jorge F. Del Teglia
 */
public class Server
///////////////////
{
  public static void main( String[] args )
  ////////////////////////////////////////
  {
    if ( args.length < 1 )
    {
      System.err.println( "** Argument missing: <host>" );  
      System.exit( -1 );
    }

    try
    {
      // Tenta iniciar a registry programaticamente. Se no consegue, ignora
      initRegistry();


      // Cria a instncia servidora.
      ChatRoomMgrImpl chatRoomMgrImpl = new ChatRoomMgrImpl( getRoomNames( args ) );

      // Registra instancia no name service.
      final String host = args[ 0 ];
      final String objectName = "//" + host + "/" + Constants.SERVICE_NAME;
      Naming.rebind( objectName, chatRoomMgrImpl );
      Helper.getInstance().log( "Server bound at " + objectName );
    }
    catch ( Exception e )
    {
      e.printStackTrace();
    }
  }


  private static void initRegistry()
  {
    try
    {
      Helper.getInstance().log( "Iniciando Registry service programaticamente." );
      LocateRegistry.createRegistry( Registry.REGISTRY_PORT );
    }
    catch ( Exception e )
    {
      Helper.getInstance().log( "No consiguiou iniciar Registry: " + e.getMessage() );
    }
  }


  /**
   * Extrai os argumentos que so os nomes das salas.
   * 
   * @param args
   */
  private static String[] getRoomNames( String[] args )
  {
    String[] roomNames = new String[ args.length - 1 ];
    for ( int i = 0; i < roomNames.length; i++ )
    {
      roomNames[ i ] = args[ i + 1 ];
    } // for

    return roomNames;
  }
} // class Server


